@@ -2,7 +2,7 @@ import logging
 import os, sys
 import warnings
 
-ROOT_PATH=__path__[0]
+ROOT_PATH=tuple(__path__)[0]
 dlog = logging.getLogger(__name__)
 dlog.propagate = False
 dlog.setLevel(logging.INFO)
@@ -32,10 +32,6 @@ try:
     from ._version import version as __version__
 except ImportError:
     __version__ = 'unkown'
-try:
-    from ._date import date as __date__
-except ImportError:
-    __date__ = 'unkown'
 
 from .submission import Submission
 from .submission import Task
@@ -58,12 +54,11 @@ from .hdfs_context import HDFSContext
 
 def info():
     """
-        Show basic information about """+NAME+""", its location and version.
+        Show basic information about dpdispatcher, its location and version.
     """
 
     print('DeepModeling\n------------')
     print('Version: ' + __version__)
-    print('Date:    ' + __date__)
     print('Path:    ' + ROOT_PATH)
     print('')
     print('Dependency')
