@@ -1,10 +1,11 @@
 
-from abc import ABCMeta, abstractmethod
-from dpdispatcher.ssh_context import SSHSession
 import json
-from dargs import Argument, Variant
-from typing import List, Union
+import shlex
 import pathlib
+from abc import ABCMeta, abstractmethod
+from typing import List, Optional, Tuple
+
+from dargs import Argument, Variant
 from dpdispatcher import dlog
 from dpdispatcher.base_context import BaseContext
 
@@ -17,7 +18,7 @@ script_template="""\
 """
 
 script_env_template="""
-REMOTE_ROOT={remote_root}
+REMOTE_ROOT=$(readlink -f {remote_root})
 echo 0 > $REMOTE_ROOT/{flag_if_job_task_fail}
 test $? -ne 0 && exit 1
 
@@ -25,6 +26,7 @@ test $? -ne 0 && exit 1
 {module_load_part}
 {source_files_part}
 {export_envs_part}
+{prepend_script_part}
 """
 
 script_command_template="""
@@ -44,6 +46,8 @@ test $? -ne 0 && exit 1
 wait
 FLAG_IF_JOB_TASK_FAIL=$(cat {flag_if_job_task_fail})
 if test $FLAG_IF_JOB_TASK_FAIL -eq 0; then touch {job_tag_finished}; else exit 1;fi
+
+{append_script_part}
 """
 
 class Machine(metaclass=ABCMeta):
@@ -57,6 +61,9 @@ class Machine(metaclass=ABCMeta):
 
     subclasses_dict = {}
     options = set()
+    # alias: for subclasses_dict key
+    # notes: this attribute can be inherited
+    alias: Tuple[str, ...] = tuple()
 
     def __new__(cls, *args, **kwargs):
         if cls is Machine:
@@ -77,7 +84,7 @@ class Machine(metaclass=ABCMeta):
     ):
         if context is None:
             assert isinstance(self, self.__class__.subclasses_dict[batch_type])
-            context = BaseContext(
+            context = BaseContext( # type: ignore
                 context_type=context_type,
                 local_root=local_root,
                 remote_root=remote_root,
@@ -101,8 +108,10 @@ class Machine(metaclass=ABCMeta):
 
     def __init_subclass__(cls, **kwargs):
         super().__init_subclass__(**kwargs)
-        cls.subclasses_dict[cls.__name__]=cls
-        cls.subclasses_dict[cls.__name__.lower()]=cls
+        alias = [cls.__name__, *cls.alias]
+        for aa in alias:
+            cls.subclasses_dict[aa]=cls
+            cls.subclasses_dict[aa.lower()]=cls
         cls.options.add(cls.__name__)
         # cls.subclasses.append(cls)
 
@@ -237,15 +246,19 @@ class Machine(metaclass=ABCMeta):
             else:
                 export_envs_part += f"export {k}={v}\n"
 
+        prepend_script = job.resources.prepend_script
+        prepend_script_part = "\n".join(prepend_script)
+
         flag_if_job_task_fail = job.job_hash + '_flag_if_job_task_fail'
 
         script_env = script_env_template.format(
             flag_if_job_task_fail=flag_if_job_task_fail,
-            remote_root=self.context.remote_root,
+            remote_root=shlex.quote(self.context.remote_root),
             module_unload_part=module_unload_part,
             module_load_part=module_load_part,
             source_files_part=source_files_part,
             export_envs_part=export_envs_part,
+            prepend_script_part=prepend_script_part
         )
         return script_env
 
@@ -261,15 +274,15 @@ class Machine(metaclass=ABCMeta):
 
             log_err_part = ""
             if task.outlog is not None:
-                log_err_part += f"1>>{task.outlog} "
+                log_err_part += f"1>>{shlex.quote(task.outlog)} "
             if task.errlog is not None:
-                log_err_part += f"2>>{task.errlog} "
+                log_err_part += f"2>>{shlex.quote(task.errlog)} "
 
             flag_if_job_task_fail = job.job_hash + '_flag_if_job_task_fail'
             single_script_command = script_command_template.format(
                 flag_if_job_task_fail=flag_if_job_task_fail,
                 command_env=command_env,
-                task_work_path=pathlib.PurePath(task.task_work_path).as_posix(),
+                task_work_path=shlex.quote(pathlib.PurePath(task.task_work_path).as_posix()),
                 command=task.command,
                 task_tag_finished=task_tag_finished,
                 log_err_part=log_err_part)
@@ -281,9 +294,14 @@ class Machine(metaclass=ABCMeta):
     def gen_script_end(self, job):
         job_tag_finished = job.job_hash + '_job_tag_finished'
         flag_if_job_task_fail = job.job_hash + '_flag_if_job_task_fail'
+
+        append_script = job.resources.append_script
+        append_script_part = "\n".join(append_script)
+        
         script_end = script_end_template.format(
             job_tag_finished=job_tag_finished,
-            flag_if_job_task_fail=flag_if_job_task_fail
+            flag_if_job_task_fail=flag_if_job_task_fail,
+            append_script_part=append_script_part
         )
         return script_end
 
@@ -357,9 +375,15 @@ class Machine(metaclass=ABCMeta):
         Argument
             resources arginfo
         """
+        alias = []
+        for aa in cls.alias:
+            alias.extend((
+                aa,
+                aa.lower(),
+            ))
         return Argument(
             cls.__name__, dict, sub_fields=cls.resources_subfields(),
-            alias=[cls.__name__.lower()]
+            alias=[cls.__name__.lower(), *alias]
         )
 
     @classmethod
