@@ -1,17 +1,21 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-from dpdispatcher.base_context import BaseContext
 import os, paramiko, tarfile, time
 import uuid
 import shutil
-from functools import lru_cache
-from glob import glob
-from dpdispatcher import dlog
-from dargs.dargs import Argument
-from typing import List
 import pathlib
 import socket
+import shlex
+from functools import lru_cache
+from glob import glob
+from typing import List
+
+import paramiko.ssh_exception
+from dargs.dargs import Argument
+
+from dpdispatcher.base_context import BaseContext
+from dpdispatcher import dlog
 # from dpdispatcher.submission import Machine
 from dpdispatcher.utils import (
     get_sha256, generate_totp, rsync,
@@ -28,7 +32,8 @@ class SSHSession (object):
                 passphrase=None,
                 timeout=10,
                 totp_secret=None,
-                tar_compress=True
+                tar_compress=True,
+                look_for_keys=True,
                 ):
 
         self.hostname = hostname
@@ -41,6 +46,7 @@ class SSHSession (object):
         self.totp_secret = totp_secret
         self.ssh = None
         self.tar_compress = tar_compress
+        self.look_for_keys = look_for_keys
         self._setup_ssh()
 
     # @classmethod
@@ -75,7 +81,7 @@ class SSHSession (object):
             if count == max_check:
                 raise RuntimeError('cannot connect ssh after %d failures at interval %d s' %
                                     (max_check, sleep_time))
-            dlog.info('connection check failed, try to reconnect to ' + self.remote_root)
+            dlog.info('connection check failed, try to reconnect to ' + self.hostname)
             self._setup_ssh()
             count += 1
             time.sleep(sleep_time)
@@ -85,6 +91,7 @@ class SSHSession (object):
             return False
         try :
             transport = self.ssh.get_transport()
+            assert transport is not None
             transport.send_ignore()
             return True
         except EOFError:
@@ -108,6 +115,7 @@ class SSHSession (object):
     #     transport = self.ssh.get_transport()
     #     transport.set_keepalive(60)
     
+    @retry(max_retry=3, sleep=1)
     def _setup_ssh(self):
         # machine = self.machine
         self.ssh = paramiko.SSHClient()
@@ -134,35 +142,76 @@ class SSHSession (object):
         ts.start_client(timeout=self.timeout)
 
         #Begin authentication; note that the username and callback are passed
-        if self.totp_secret:
-            ts.auth_interactive(self.username, self.inter_handler)
-        else:
-            key_path = os.path.join(os.path.expanduser("~"), ".ssh", "id_rsa")
-            if self.key_filename:
-                key_path = os.path.abspath(self.key_filename)
-            key = None
+        key = None
+        key_ok = False
+        key_error = None
+        keyfiles = []
+        if self.key_filename:
+            key_path = os.path.abspath(self.key_filename)
             if os.path.exists(key_path):
+                for pkey_class in (
+                    paramiko.RSAKey, 
+                    paramiko.DSSKey, 
+                    paramiko.ECDSAKey, 
+                    paramiko.Ed25519Key
+                ):
+                    try:
+                        # passing empty passphrase would not raise error.
+                        key = pkey_class.from_private_key_file(key_path, self.passphrase)
+                    except paramiko.SSHException as e:
+                        pass
+                    if key is not None:
+                        break
+        elif self.look_for_keys:
+            for keytype, name in [
+                (paramiko.RSAKey, "rsa"),
+                (paramiko.DSSKey, "dsa"),
+                (paramiko.ECDSAKey, "ecdsa"),
+                (paramiko.Ed25519Key, "ed25519"),
+            ]:
+                for directory in [".ssh", "ssh"]:
+                    full_path = os.path.join(
+                        os.path.expanduser("~"),
+                        directory, f"id_{name}"
+                    )
+                    if os.path.isfile(full_path):
+                        keyfiles.append((keytype, full_path))
+                        # TODO: supporting cert
+            for pkey_class, filename in keyfiles:
                 try:
-                    key = paramiko.RSAKey.from_private_key_file(key_path)
-                except paramiko.PasswordRequiredException:
-                    key = paramiko.RSAKey.from_private_key_file(key_path, self.passphrase)
-            if key:
-                try:
-                    ts.auth_publickey(self.username, key)
-                except paramiko.ssh_exception.AuthenticationException:
-                    if self.password:
-                        ts.auth_password(self.username, self.password)
-                    else:
-                        raise RuntimeError("Authentication failed, try to provide password")
-            elif self.password:
-                ts.auth_password(self.username, self.password)
+                    key = pkey_class.from_private_key_file(filename, self.passphrase)
+                except paramiko.SSHException as e:
+                    pass
+                if key is not None:
+                    break
+        
+        if key is not None:
+            try:
+                ts.auth_publickey(self.username, key)
+            except paramiko.ssh_exception.AuthenticationException as e:
+                key_error = e
             else:
-                raise RuntimeError("Please provide at least one form of authentication")
+                key_ok = True
+        if self.totp_secret is not None:
+            try:
+                ts.auth_interactive(self.username, self.inter_handler)
+            except paramiko.ssh_exception.AuthenticationException:
+                # since the asynchrony of interactive authentication, one addtional try is added
+                # retry for up to 3 times
+                raise RetrySignal("Authentication failed")
+        elif key_ok:
+            pass
+        elif self.password is not None:
+            ts.auth_password(self.username, self.password)
+        elif key_error is not None:
+            raise RuntimeError("Authentication failed, try to provide password") from key_error
+        else:
+            raise RuntimeError("Please provide at least one form of authentication")
         assert(ts.is_active())
         #Opening a session creates a channel along the socket to the server
         ts.open_session(timeout=self.timeout)
         ts.set_keepalive(60)
-        self.ssh._transport = ts
+        self.ssh._transport = ts # type: ignore
         # reset sftp
         self._sftp = None
                         
@@ -199,10 +248,11 @@ class SSHSession (object):
                 resp.append(self.username)
             elif "password" in pr_str:
                 resp.append(self.password)
-            elif "verification" in pr_str and self.totp_secret:
+            elif "verification" in pr_str or "token" in pr_str and self.totp_secret is not None:
+                assert self.totp_secret is not None
                 resp.append(generate_totp(self.totp_secret))
 
-        return tuple(resp)  #Convert the response list to a tuple and return it
+        return resp
 
     def get_ssh_client(self) :
         return self.ssh
@@ -211,11 +261,13 @@ class SSHSession (object):
     #     return self.remote_root
 
     def close(self) :
+        assert self.ssh is not None
         self.ssh.close()
 
     @retry(sleep=1)
     def exec_command(self, cmd):
         """Calling self.ssh.exec_command but has an exception check."""
+        assert self.ssh is not None
         try:
             return self.ssh.exec_command(cmd)
         except (paramiko.ssh_exception.SSHException, socket.timeout) as e:
@@ -229,6 +281,7 @@ class SSHSession (object):
     def sftp(self):
         """Returns sftp. Open a new one if not existing."""
         if self._sftp is None:
+            assert self.ssh is not None
             self.ensure_alive()
             self._sftp = self.ssh.open_sftp()
         return self._sftp
@@ -247,6 +300,7 @@ class SSHSession (object):
         doc_totp_secret = 'Time-based one time password secret. It should be a base32-encoded string' \
                           ' extracted from the 2D code.'
         doc_tar_compress = 'The archive will be compressed in upload and download if it is True. If not, compression will be skipped.'
+        doc_look_for_keys = "enable searching for discoverable private key files in ~/.ssh/"
 
         ssh_remote_profile_args = [
             Argument("hostname", str, optional=False, doc=doc_hostname),
@@ -258,25 +312,27 @@ class SSHSession (object):
             Argument("timeout", int, optional=True, default=10, doc=doc_timeout),
             Argument("totp_secret", str, optional=True, default=None, doc=doc_totp_secret),
             Argument("tar_compress", bool, optional=True, default=True, doc = doc_tar_compress),
+            Argument("look_for_keys", bool, optional=True, default=True, doc=doc_look_for_keys),
         ]
         ssh_remote_profile_format = Argument("ssh_session", dict, ssh_remote_profile_args)
         return ssh_remote_profile_format
         
     def put(self, from_f, to_f):
         if self.rsync_available:
-            return rsync(from_f, self.remote + ":" + to_f)
+            return rsync(from_f, self.remote + ":" + to_f, port=self.port, key_filename=self.key_filename, timeout=self.timeout)
         return self.sftp.put(from_f, to_f)
 
     def get(self, from_f, to_f):
         if self.rsync_available:
-            return rsync(self.remote + ":" + from_f, to_f)
+            return rsync(self.remote + ":" + from_f, to_f, port=self.port, key_filename=self.key_filename, timeout=self.timeout)
         return self.sftp.get(from_f, to_f)
 
     @property
     @lru_cache(maxsize=None)
     def rsync_available(self) -> bool:
         return (shutil.which("rsync") is not None and self.password is None
-            and self.port == 22 and self.key_filename is None
+            and self.exec_command("rsync --version")[1].channel.recv_exit_status() == 0
+            and self.totp_secret is None
             and self.passphrase is None)
 
     @property
@@ -300,6 +356,7 @@ class SSHContext(BaseContext):
         assert os.path.isabs(remote_root), f"remote_root must be a abspath"
         self.temp_remote_root = remote_root
         self.remote_profile = remote_profile
+        self.remote_root = None
 
         # self.job_uuid = None
         self.clean_asynchronously = clean_asynchronously
@@ -308,7 +365,7 @@ class SSHContext(BaseContext):
         #    self.job_uuid=job_uuid
         # else:
         #    self.job_uuid = str(uuid.uuid4())
-        self.ssh_session = SSHSession(**remote_profile)
+        self.ssh_session = SSHSession(**remote_profile) # type: ignore
         # self.temp_remote_root = os.path.join(self.ssh_session.get_session_root())
         self.ssh_session.ensure_alive()
         try:
@@ -365,10 +422,18 @@ class SSHContext(BaseContext):
         return self.remote_root
 
     def bind_submission(self, submission):
+        assert self.ssh_session is not None
+        assert self.ssh_session.ssh is not None
         self.submission = submission
         self.local_root = pathlib.PurePath(os.path.join(self.temp_local_root, submission.work_base)).as_posix()
+        old_remote_root = self.remote_root
         # self.remote_root = os.path.join(self.temp_remote_root, self.submission.submission_hash, self.submission.work_base )
         self.remote_root = pathlib.PurePath(os.path.join(self.temp_remote_root, self.submission.submission_hash)).as_posix()
+        # move the working directory if remote_root changes
+        if old_remote_root is not None and old_remote_root != self.remote_root \
+            and self.check_file_exists(old_remote_root) \
+            and not self.check_file_exists(self.remote_root):
+            self.block_checkcall(f"mv {shlex.quote(old_remote_root)} {shlex.quote(self.remote_root)}")
 
         sftp = self.ssh_session.ssh.open_sftp()
         try:
@@ -413,6 +478,7 @@ class SSHContext(BaseContext):
                submission,
                # local_up_files,
                dereference = True) :
+        assert self.remote_root is not None
         dlog.info(f'remote path: {self.remote_root}')
         # remote_cwd = 
         self.ssh_session.sftp.chdir(self.temp_remote_root)
@@ -425,14 +491,17 @@ class SSHContext(BaseContext):
                 recover = True
         self.ssh_session.sftp.chdir(None)
 
-        cwd = os.getcwd()
-        os.chdir(self.local_root) 
         file_list = []
         directory_list = []
         for task in submission.belonging_tasks:
             directory_list.append(task.task_work_path)
         #     file_list.append(ii)
-            self._walk_directory(task.forward_files, task.task_work_path, file_list, directory_list)
+            self._walk_directory(
+                task.forward_files, 
+                os.path.join(self.local_root, task.task_work_path), 
+                file_list, 
+                directory_list
+            )
         self._walk_directory(submission.forward_common_files, self.local_root, file_list, directory_list)
 
         # check if the same file exists on the remote file
@@ -449,7 +518,7 @@ class SSHContext(BaseContext):
             self.write_file(sha256_file, "\n".join(sha256_list))
             # check sha256
             # `:` means pass: https://stackoverflow.com/a/2421592/9567349
-            _, stdout, _ = self.block_checkcall("sha256sum -c %s --quiet >.sha256sum_stdout 2>/dev/null || :" % sha256_file)
+            _, stdout, _ = self.block_checkcall("sha256sum -c %s --quiet >.sha256sum_stdout 2>/dev/null || :" % shlex.quote(sha256_file))
             self.sftp.remove(sha256_file)
             # regenerate file list
             file_list = []
@@ -462,7 +531,6 @@ class SSHContext(BaseContext):
             file_list = [os.path.relpath(jj, self.local_root) for jj in file_list] 
 
         self._put_files(file_list, dereference = dereference, directories=directory_list, tar_compress = self.remote_profile.get('tar_compress', None))
-        os.chdir(cwd)
 
     def download(self, 
                  submission,
@@ -472,8 +540,6 @@ class SSHContext(BaseContext):
                  mark_failure = True,
                  back_error=False) :
         self.ssh_session.ensure_alive()
-        cwd = os.getcwd()
-        os.chdir(self.local_root) 
         file_list = []
         # for ii in job_dirs :
         for task in submission.belonging_tasks :
@@ -494,7 +560,6 @@ class SSHContext(BaseContext):
         file_list.extend(submission.backward_common_files)
         if len(file_list) > 0:
             self._get_files(file_list, tar_compress = self.remote_profile.get('tar_compress', None))
-        os.chdir(cwd)
         
     def block_checkcall(self, 
                         cmd,
@@ -510,10 +575,11 @@ class SSHContext(BaseContext):
         asynchronously: bool, optional, default=False
             Run command asynchronously. If True, `nohup` will be used to run the command.
         """
+        assert self.remote_root is not None
         self.ssh_session.ensure_alive()
         if asynchronously:
             cmd = "nohup %s >/dev/null &" % cmd
-        stdin, stdout, stderr = self.ssh_session.exec_command(('cd %s ;' % self.remote_root) + cmd)
+        stdin, stdout, stderr = self.ssh_session.exec_command(('cd %s ;' % shlex.quote(self.remote_root)) + cmd)
         exit_status = stdout.channel.recv_exit_status() 
         if exit_status != 0:
             raise RuntimeError("Get error code %d in calling %s through ssh with job: %s . message: %s" %
@@ -522,8 +588,9 @@ class SSHContext(BaseContext):
 
     def block_call(self, 
                    cmd) :
+        assert self.remote_root is not None
         self.ssh_session.ensure_alive()
-        stdin, stdout, stderr = self.ssh_session.exec_command(('cd %s ;' % self.remote_root) + cmd)
+        stdin, stdout, stderr = self.ssh_session.exec_command(('cd %s ;' % shlex.quote(self.remote_root)) + cmd)
         exit_status = stdout.channel.recv_exit_status() 
         return exit_status, stdin, stdout, stderr
 
@@ -532,6 +599,7 @@ class SSHContext(BaseContext):
         self._rmtree(self.remote_root)
 
     def write_file(self, fname, write_str):
+        assert self.remote_root is not None
         self.ssh_session.ensure_alive()
         fname = pathlib.PurePath(os.path.join(self.remote_root, fname)).as_posix()
         # to prevent old file from being overwritten but cancelled, create a temporary file first
@@ -539,15 +607,17 @@ class SSHContext(BaseContext):
         with self.sftp.open(fname + "~", 'w') as fp :
             fp.write(write_str)
         # sftp.rename may throw OSError
-        self.block_checkcall("mv %s %s" % (fname + "~", fname))
+        self.block_checkcall("mv %s %s" % (shlex.quote(fname + "~"), shlex.quote(fname)))
 
     def read_file(self, fname):
+        assert self.remote_root is not None
         self.ssh_session.ensure_alive()
         with self.sftp.open(pathlib.PurePath(os.path.join(self.remote_root, fname)).as_posix(), 'r') as fp:
             ret = fp.read().decode('utf-8')
         return ret
 
     def check_file_exists(self, fname):
+        assert self.remote_root is not None
         self.ssh_session.ensure_alive()
         try:
             self.sftp.stat(pathlib.PurePath(os.path.join(self.remote_root, fname)).as_posix())
@@ -590,7 +660,7 @@ class SSHContext(BaseContext):
         # In some supercomputers, it's very slow to remove large numbers of files
         # (e.g. directory containing trajectory) due to bad I/O performance.
         # So an asynchronously option is provided.
-        self.block_checkcall('rm -rf %s' % remotepath, asynchronously=self.clean_asynchronously)
+        self.block_checkcall('rm -rf %s' % shlex.quote(remotepath), asynchronously=self.clean_asynchronously)
 
     def _put_files(self,
                    files,
@@ -615,6 +685,7 @@ class SSHContext(BaseContext):
             If tar_compress is True, compress the archive using gzip
             It it is False, then it is uncompressed
         """
+        assert self.remote_root is not None
         of_suffix = '.tgz'
         tarfile_mode = "w:gz"
         kwargs = {'compresslevel': 6}
@@ -625,17 +696,17 @@ class SSHContext(BaseContext):
                     
         of = self.submission.submission_hash + of_suffix
         # local tar
-        cwd = os.getcwd()
-        os.chdir(self.local_root)
-        if os.path.isfile(of) :
-            os.remove(of)
-        with tarfile.open(of, tarfile_mode, dereference = dereference, **kwargs) as tar:
-            for ii in files :
-                tar.add(ii)
+        if os.path.isfile(os.path.join(self.local_root, of)):
+            os.remove(os.path.join(self.local_root, of))
+        with tarfile.open(os.path.join(self.local_root, of), tarfile_mode, dereference = dereference, **kwargs) as tar:
+            # avoid compressing duplicated files or directories
+            for ii in set(files) :
+                ii_full = os.path.join(self.local_root, ii)
+                tar.add(ii_full, arcname=ii)
             if directories is not None:
-                for ii in directories:
-                    tar.add(ii, recursive=False)
-        os.chdir(cwd)
+                for ii in set(directories):
+                    ii_full = os.path.join(self.local_root, ii)
+                    tar.add(ii_full, arcname=ii, recursive=False)
         self.ssh_session.ensure_alive()
         try:
             self.sftp.mkdir(self.remote_root)
@@ -657,7 +728,10 @@ class SSHContext(BaseContext):
     def _get_files(self, 
                    files,
                    tar_compress = True) :
-        
+        assert self.remote_root is not None
+        # avoid compressing duplicated files
+        files = list(set(files))
+
         of_suffix = '.tar.gz'
         tarfile_mode = "r:gz"
         tar_command = 'czfh'
@@ -673,11 +747,11 @@ class SSHContext(BaseContext):
         per_nfile = 100
         ntar = len(files) // per_nfile + 1
         if ntar <= 1:
-            self.block_checkcall('tar %s %s %s' % (tar_command, of, " ".join(files)))
+            self.block_checkcall('tar %s %s %s' % (tar_command, shlex.quote(of), " ".join([shlex.quote(file) for file in files])))
         else:
             file_list_file = os.path.join(self.remote_root, ".tmp.tar." + str(uuid.uuid4()))
             self.write_file(file_list_file, "\n".join(files))
-            self.block_checkcall('tar %s %s -T %s' % (tar_command, of, file_list_file))
+            self.block_checkcall('tar %s %s -T %s' % (tar_command, shlex.quote(of), shlex.quote(file_list_file)))
         # trans
         from_f = pathlib.PurePath(os.path.join(self.remote_root, of)).as_posix()
         to_f = pathlib.PurePath(os.path.join(self.local_root, of)).as_posix()
@@ -685,11 +759,8 @@ class SSHContext(BaseContext):
             os.remove(to_f)
         self.ssh_session.get(from_f, to_f)
         # extract
-        cwd = os.getcwd()
-        os.chdir(self.local_root)
-        with tarfile.open(of, mode = tarfile_mode) as tar:
-            tar.extractall()
-        os.chdir(cwd)    
+        with tarfile.open(to_f, mode = tarfile_mode) as tar:
+            tar.extractall(path=self.local_root)
         # cleanup
         os.remove(to_f)
         self.sftp.remove(from_f)
