@@ -113,6 +113,7 @@ class Submission(object):
         submission_dict : dict
             the dictionary converted from the Submission class instance
         """
+        assert self.resources is not None
         submission_dict = {}
         # if if_none_local_root:
         #     submission_dict['local_root'] = None
@@ -172,6 +173,7 @@ class Submission(object):
         Forth, wait until the tasks in the submission finished and download the result file to local directory.
         if exit_on_submit is True, submission will exit.
         """
+        assert self.resources is not None
         if not self.belonging_jobs:
             self.generate_jobs()
         self.try_recover_from_json()
@@ -315,6 +317,7 @@ class Submission(object):
         Why we randomly shuffle the tasks is under the consideration of load balance.
         The random seed is a constant (to be concrete, 42). And this insures that the jobs are equal when we re-run the program.
         """
+        assert self.resources is not None
         if self.belonging_jobs:
             raise RuntimeError(f'Can not generate jobs when submission.belonging_jobs is not empty. debug:{self}')
         group_size = self.resources.group_size
@@ -592,6 +595,7 @@ class Job(object):
         this method will not submit or resubmit the jobs if the job is unsubmitted.
         """
         dlog.debug(f"debug:query database; self.job_hash:{self.job_hash}; self.job_id:{self.job_id}")
+        assert self.machine is not None
         job_state = self.machine.check_status(self)
         self.job_state = job_state
 
@@ -661,6 +665,7 @@ class Job(object):
         self.job_id = job_id
 
     def submit_job(self):
+        assert self.machine is not None
         job_id = self.machine.do_submit(self)
         self.register_job_id(job_id)
         if job_id:
@@ -670,6 +675,7 @@ class Job(object):
 
     def job_to_json(self):
         write_str = json.dumps(self.serialize(), indent=2, default=str)
+        assert self.machine is not None
         self.machine.context.write_file(self.job_hash + '_job.json', write_str=write_str)
 
 
@@ -721,6 +727,8 @@ class Resources(object):
                 module_list=[],
                 source_list=[],
                 envs={},
+                prepend_script=[],
+                append_script=[],
                 wait_time=0,
                 **kwargs):
         self.number_node = number_node
@@ -738,6 +746,8 @@ class Resources(object):
         self.module_list = module_list
         self.source_list = source_list
         self.envs = envs
+        self.prepend_script = prepend_script
+        self.append_script = append_script
         self.wait_time = wait_time
         # self.if_cuda_multi_devices = if_cuda_multi_devices
 
@@ -779,6 +789,8 @@ class Resources(object):
         resources_dict['module_list'] = self.module_list
         resources_dict['source_list'] = self.source_list
         resources_dict['envs'] = self.envs
+        resources_dict['prepend_script'] = self.prepend_script
+        resources_dict['append_script'] = self.append_script
         resources_dict['wait_time'] = self.wait_time
         resources_dict['kwargs'] = self.kwargs
         return resources_dict
@@ -798,6 +810,8 @@ class Resources(object):
                         module_list=resources_dict.get('module_list', []),
                         source_list=resources_dict.get('source_list', []),
                         envs=resources_dict.get('envs', {}),
+                        prepend_script=resources_dict.get('prepend_script', []),
+                        append_script=resources_dict.get('append_script', []),
                         wait_time=resources_dict.get('wait_time', 0),
                         **resources_dict.get('kwargs', {})
                         )
@@ -836,6 +850,8 @@ class Resources(object):
         doc_module_unload_list = 'The modules to be unloaded on HPC system before submitting jobs'
         doc_module_list = 'The modules to be loaded on HPC system before submitting jobs'
         doc_envs = 'The environment variables to be exported on before submitting jobs'
+        doc_prepend_script = 'Optional script run before jobs submitted.'
+        doc_append_script = 'Optional script run after jobs submitted.'
         doc_wait_time = 'The waitting time in second after a single `task` submitted'
         doc_if_cuda_multi_devices = ('If there are multiple nvidia GPUS on the node, and we want to assign the tasks to different GPUS.'
             'If true, dpdispatcher will manually export environment variable CUDA_VISIBLE_DEVICES to different task.'
@@ -865,6 +881,8 @@ class Resources(object):
             Argument("module_unload_list", list, optional=True, doc=doc_module_unload_list, default=[]),
             Argument("module_list", list, optional=True, doc=doc_module_list, default=[]),
             Argument("envs", dict, optional=True, doc=doc_envs, default={}),
+            Argument("prepend_script", list, optional=True, doc=doc_prepend_script, default=[]),
+            Argument("append_script", list, optional=True, doc=doc_append_script, default=[]),
             Argument("wait_time", [int, float], optional=True, doc=doc_wait_time, default=0)
         ]
 
