@@ -4,7 +4,7 @@ import struct
 import hmac
 import base64
 import subprocess
-from typing import Callable, Union
+from typing import Iterable, Optional, Callable, Type, Union
 
 from dpdispatcher import dlog
 
@@ -32,9 +32,9 @@ def get_sha256(filename):
     return sha256
 
 def hotp(key: str, period: int, token_length: int=6, digest='sha1'):
-    key = base64.b32decode(key.upper() + '=' * ((8 - len(key)) % 8))
-    period = struct.pack('>Q', period)
-    mac = hmac.new(key, period, digest).digest()
+    key_ = base64.b32decode(key.upper() + '=' * ((8 - len(key)) % 8))
+    period_ = struct.pack('>Q', period)
+    mac = hmac.new(key_, period_, digest).digest()
     offset = mac[-1] & 0x0f
     binary = struct.unpack('>L', mac[offset:offset+4])[0] & 0x7fffffff
     return str(binary)[-token_length:].zfill(token_length)
@@ -74,7 +74,7 @@ def run_cmd_with_all_output(cmd, shell=True):
     return (ret, out, err)
 
 
-def rsync(from_file: str, to_file: str):
+def rsync(from_file: str, to_file: str, port: int = 22, key_filename: Optional[str] = None, timeout: Union[int, float] = 10):
     """Call rsync to transfer files.
     
     Parameters
@@ -83,17 +83,40 @@ def rsync(from_file: str, to_file: str):
         SRC
     to_file: str
         DEST
+    port : int, default=22
+        port for ssh
+    key_filename : str, optional
+        identity file name
+    timeout : int, default=10
+        timeout for ssh
     
     Raises
     ------
     RuntimeError
         when return code is not 0
     """
+    ssh_cmd = [
+        'ssh',
+        '-o',
+        'ConnectTimeout=' + str(timeout),
+        '-o',
+        'BatchMode=yes',
+        '-o',
+        'StrictHostKeyChecking=no',
+        '-p',
+        str(port),
+        '-q',
+    ]
+    if key_filename is not None:
+        ssh_cmd.extend(['-i', key_filename])
     cmd = [
         'rsync',
         # -a: archieve
         # -z: compress
         '-az',
+        '-e',
+        " ".join(ssh_cmd),
+        '-q',
         from_file,
         to_file,
     ]
@@ -106,7 +129,7 @@ class RetrySignal(Exception):
     """Exception to give a signal to retry the function."""
 
 
-def retry(max_retry: int = 3, sleep: Union[int, float] = 60, catch_exception: BaseException = RetrySignal) -> Callable:
+def retry(max_retry: int = 3, sleep: Union[int, float] = 60, catch_exception: Type[BaseException] = RetrySignal) -> Callable:
     """Retry the function until it succeeds or fails for certain times.
 
     Parameters
@@ -137,7 +160,7 @@ def retry(max_retry: int = 3, sleep: Union[int, float] = 60, catch_exception: Ba
             while max_retry is None or current_retry < max_retry:
                 try:
                     return func(*args, **kwargs)
-                except catch_exception as e:
+                except (catch_exception,) as e:
                     errors.append(e)
                     dlog.exception("Failed to run %s: %s", func.__name__, e)
                     # sleep certain seconds
